Claw = {}

if select(2, UnitClass("player")) ~= "DRUID" then
    DisableAddOn("Claw")
    return
end

SLASH_Claw1, SLASH_Claw2 = "/claw", "/clawesome"

local function InitializeVariables()
	Claw.locked = Claw.locked == nil and false or Claw.locked
	Claw.scale = Claw.scale == nil and 1 or Claw.scale
	Claw.frequency = Claw.frequency == nil and 0.05 or Claw.frequency
	Claw.latency = Claw.latency == nil and 0.3 or Claw.latency
	Claw.previous = Claw.previous == nil and true or Claw.previous
	Claw.offgcd = Claw.offgcd == nil and true or Claw.offgcd
	Claw.misseffect = Claw.misseffect == nil and true or Claw.misseffect
	Claw.glow = Claw.glow == nil and true or Claw.glow
	Claw.charge = Claw.charge == nil and false or Claw.charge
	Claw.berserk = Claw.berserk == nil and false or Claw.berserk
	Claw.tfberserk = Claw.tfberserk == nil and false or Claw.tfberserk
	Claw.roar = Claw.roar == nil and true or Claw.roar
	Claw.motw = Claw.motw == nil and false or Claw.motw
end

local events, targetHealthArray = {}, {}
local lastAbilityCalculation, GCD, currentTime, ripExtendsLeft, T13SetPieces = 0, 0, 0, 0, 0
local hostileTargetted, playerHasBloodlettingGlyph = false, false

local clawPanel = CreateFrame("Frame", "clawPanel", UIParent)
clawPanel:SetPoint("CENTER", 0, -169)
clawPanel:SetFrameStrata("BACKGROUND")
clawPanel:Hide()
clawPanel:RegisterForDrag("LeftButton")
clawPanel:SetScript("OnDragStart", clawPanel.StartMoving)
clawPanel:SetScript("OnDragStop", clawPanel.StopMovingOrSizing)
clawPanel:SetMovable(true)
clawPanel.icon = clawPanel:CreateTexture(nil, "BACKGROUND")
clawPanel.icon:SetAllPoints(clawPanel)
clawPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawPanel.border = clawPanel:CreateTexture(nil, "BORDER")
clawPanel.border:SetAllPoints(clawPanel)
clawPanel.border:SetTexture("Interface\\AddOns\\Claw\\border.blp")
clawPanel.border:Hide()
local clawPreviousPanel = CreateFrame("Frame", "clawPreviousPanel", UIParent)
clawPreviousPanel:SetPoint("BOTTOMRIGHT", clawPanel, "BOTTOMLEFT", -10, -5)
clawPreviousPanel:SetFrameStrata("BACKGROUND")
clawPreviousPanel:Hide()
clawPreviousPanel:RegisterForDrag("LeftButton")
clawPreviousPanel:SetScript("OnDragStart", clawPreviousPanel.StartMoving)
clawPreviousPanel:SetScript("OnDragStop", clawPreviousPanel.StopMovingOrSizing)
clawPreviousPanel:SetMovable(true)
clawPreviousPanel.icon = clawPreviousPanel:CreateTexture(nil, "BACKGROUND")
clawPreviousPanel.icon:SetAllPoints(clawPreviousPanel)
clawPreviousPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawPreviousPanel.border = clawPreviousPanel:CreateTexture(nil, "BORDER")
clawPreviousPanel.border:SetAllPoints(clawPreviousPanel)
clawPreviousPanel.border:SetTexture("Interface\\AddOns\\Claw\\border.blp")
local clawOffGCDPanel = CreateFrame("Frame", "clawOffGCDPanel", UIParent)
clawOffGCDPanel:SetPoint("BOTTOMLEFT", clawPanel, "BOTTOMRIGHT", 10, -5)
clawOffGCDPanel:SetFrameStrata("BACKGROUND")
clawOffGCDPanel:Hide()
clawOffGCDPanel:RegisterForDrag("LeftButton")
clawOffGCDPanel:SetScript("OnDragStart", clawOffGCDPanel.StartMoving)
clawOffGCDPanel:SetScript("OnDragStop", clawOffGCDPanel.StopMovingOrSizing)
clawOffGCDPanel:SetMovable(true)
clawOffGCDPanel.icon = clawOffGCDPanel:CreateTexture(nil, "BACKGROUND")
clawOffGCDPanel.icon:SetAllPoints(clawOffGCDPanel)
clawOffGCDPanel.icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
clawOffGCDPanel.border = clawOffGCDPanel:CreateTexture(nil, "BORDER")
clawOffGCDPanel.border:SetAllPoints(clawOffGCDPanel)
clawOffGCDPanel.border:SetTexture("Interface\\AddOns\\Claw\\border.blp")

local Ability, abilityList = {}, {}
Ability.__index = Ability

function Ability.add(spellId, buff, playerCast)
    local ability = {}
    setmetatable(ability, Ability)
	ability.spellId = spellId
    ability.buff = buff or false
    ability.playerCast = playerCast or false
    ability.name = GetSpellInfo(spellId)
    ability.icon = select(3, GetSpellInfo(spellId))
	if spellId ~= 0 then
		abilityList[#abilityList + 1] = ability
	end
    return ability
end

function Ability:ready(seconds)
	return self:cooldown() <= (seconds or 0)
end

function Ability:remains()
    return max((select(7, UnitAura(self.buff and "player" or "target", self.name, nil, (self.buff and "HELPFUL" or "HARMFUL") .. (self.playerCast and "|PLAYER" or ""))) or 0) - currentTime - Claw.latency, 0)
end

function Ability:up()
    return select(6, UnitAura(self.buff and "player" or "target", self.name, nil, (self.buff and "HELPFUL" or "HARMFUL") .. (self.playerCast and "|PLAYER" or ""))) == 0 and true or self:remains() > 0
end

function Ability:inRange()
    return IsSpellInRange(self.name, "target") == 1
end

function Ability:cooldown()
    local startTime, duration = GetSpellCooldown(self.spellId)
    return max(duration - (currentTime - startTime) - Claw.latency, 0)
end

function Ability:stack()
    return select(4, UnitAura(self.buff and "player" or "target", self.name, nil, (self.buff and "HELPFUL" or "HARMFUL") .. (self.playerCast and "|PLAYER" or ""))) or 0
end

function Ability:known()
	return GetSpellBookItemInfo(self.name, "player") and true or false
end

function Ability:checkButtonAndGlow(button, on)
	if button then
		local actionType, spellIdOrMacro = GetActionInfo(button.action)
		if actionType then
			if (actionType == "spell" and spellIdOrMacro == self.spellId) or (actionType == "macro" and (select(2, GetMacroInfo(spellIdOrMacro))) == self.icon) then
				if on then
					ActionButton_ShowOverlayGlow(button)
				else
					ActionButton_HideOverlayGlow(button)
				end
			end
		end
	end
end

function Ability:glow(on)
	for i = 1, 12 do
		self:checkButtonAndGlow(_G["ActionButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarLeftButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarRightButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarBottomLeftButton" .. i], on)
		self:checkButtonAndGlow(_G["MultiBarBottomRightButton" .. i], on)
	end
	if Dominos then
		for i = 1, 60 do self:checkButtonAndGlow(_G["DominosActionButton" .. i], on) end
	end
end

-- Druid abilities
local Berserk = Ability.add(50334, true, true)
local FaerieFireFeral = Ability.add(16857, false, false)
local OmenOfClarity = Ability.add(16870, true, true)
-- Cat abilities
local CatForm = Ability.add(768, true, true)
local FeralChargeCat = Ability.add(49376, false, true)
local FerociousBite = Ability.add(22568, false, true)
local MangleCat = Ability.add(33876, false, false)
local Rake = Ability.add(1822, false, true)
local Ravage = Ability.add(6785, false, true)
local Rip = Ability.add(1079, false, true)
local SavageRoar = Ability.add(52610, true, true)
local Shred = Ability.add(5221, false, true)
local Stampede = Ability.add(81022, true, true)
local TigersFury = Ability.add(5217, true, true)
-- Bear abilities
local BearForm = Ability.add(5487, true, true)
local DemoralizingRoar = Ability.add(99, false, false)
local Enrage = Ability.add(5229, true, true)
local FeralChargeBear = Ability.add(16979, false, true)
local Lacerate = Ability.add(33745, false, true)
local MangleBear = Ability.add(33878, false, false)
local Maul = Ability.add(6807, false, true)
local Pulverize = Ability.add(80313, true, true)
local Thrash = Ability.add(77758, false, true)
--   10% stat buff
local BlessingOfKings = Ability.add(79063, true, false)
local MarkOfTheWild = Ability.add(1126, true, false)
--   10% damage reduction
local DemoralizingShout = Ability.add(1160, false, false)
local ScarletFever = Ability.add(81130, false, false)
local Vindication = Ability.add(26017, false, false)
--   12% armor reduction
local ExposeArmor = Ability.add(8647, false, false)
local FaerieFire = Ability.add(91565, false, false)
local SunderArmor = Ability.add(7386, false, false)
--   30% bleed damage
local Hemorrhage = Ability.add(16511, false, false)
local PetGore = Ability.add(35290, false, false)
local PetStampede = Ability.add(57386, false, false)
local PetTendonRip = Ability.add(50271, false, false)
local Trauma = Ability.add(46857, false, false)
-- Non-existent spell
local NoSpell = Ability.add(0, false, false)

local abilityNow, oldAbilityNow, abilityPrevious, abilityOffGCD = NoSpell, NoSpell, NoSpell, NoSpell

local function ComboPoints()
    return GetComboPoints("player", "target")
end

local function Energy()
    return CatForm:up() and min(UnitPower("player") + (GetPowerRegen() * Claw.latency), TigersFury:up() and 120 or 100) or 100
end

local function Rage()
    return BearForm:up() and UnitPower("player") or 0
end

local function TargetHealthPercentage()
    return UnitHealth("target") / UnitHealthMax("target") * 100
end

local function TargetTimeToDie()
    local targetTimeToDie = UnitHealth("target") / ((targetHealthArray[1] - targetHealthArray[#targetHealthArray]) / 3) - Claw.latency
    return targetTimeToDie >= 0 and targetTimeToDie or 600
end

local function TargetIsBoss()
	return UnitLevel("target") == -1 or (UnitLevel("target") >= UnitLevel("player") + 2 and not UnitInRaid("player"))
end

local function StatBuffActive()
	return BlessingOfKings:up() or MarkOfTheWild:up()
end

local function BleedDebuffActive()
	return Hemorrhage:up() or PetGore:up() or PetStampede:up() or PetTendonRip:up() or Trauma:up()
end

local function ArmorReductionDebuffActive()
	return ExposeArmor:up() or SunderArmor:up()
end

local function CalculateBestChoiceAbilityCat()
    if TigersFury:ready() and Energy() <= 35 and not Berserk:up() and not (Stampede:up() and T13SetPieces >= 4) then
        abilityOffGCD = TigersFury
	else
		abilityOffGCD = NoSpell
	end
    if not Shred:inRange() and FaerieFireFeral:ready() and FaerieFire:stack() < 3 and not ArmorReductionDebuffActive() then
        return FaerieFireFeral
    elseif Claw.charge and FeralChargeCat:ready() and FeralChargeCat:inRange() then
        return FeralChargeCat
    elseif TargetTimeToDie() <= 2 and ComboPoints() >= 1 and Energy() >= 25 then
        return FerociousBite
	elseif Claw.tfberserk and TigersFury:ready() and Berserk:ready() and Energy() <= 35 and not (Stampede:up() and T13SetPieces >= 4) then
		return Berserk
    elseif not Claw.offgcd and TigersFury:ready() and Energy() <= 35 and not (Berserk:up() or (Stampede:up() and T13SetPieces >= 4)) then
        return TigersFury
    elseif (Claw.berserk or TargetIsBoss()) and Berserk:ready() and (TigersFury:up() or (TargetTimeToDie() < 25 and not TigersFury:ready(6))) then
        return Berserk
    elseif FaerieFireFeral:ready() and FaerieFire:stack() < 3 and not ArmorReductionDebuffActive() then
        return FaerieFireFeral
    elseif Stampede:up() and Stampede:remains() <= 1 then
        return Ravage
    elseif MangleCat:remains() <= 2 and not (Trauma:up() or Hemorrhage:up() or PetGore:up() or PetStampede:up() or PetTendonRip:up()) then
        return MangleCat
    elseif ((ComboPoints() >= 1 and Rip:up() and Rip:remains() <= 2.1) or (ComboPoints() == 5 and Rip:up())) and TargetHealthPercentage() <= (T13SetPieces >= 2 and 60 or 25) then
        return FerociousBite
    elseif playerHasBloodlettingGlyph and ripExtendsLeft >= 1 and Rip:up() and Rip:remains() <= 4 and TargetHealthPercentage() > (T13SetPieces >= 2 and 60 or 25) then
        return Shred
    elseif ComboPoints() == 5 and TargetTimeToDie() >= 6 and Rip:remains() <= 2 and (Berserk:up() or Rip:remains() <= TigersFury:cooldown()) then
        return Rip
    elseif ComboPoints() == 5 and Rip:remains() >= 5 and SavageRoar:remains() >= 3 and Berserk:up() then
        return FerociousBite
    elseif (TargetTimeToDie() >= 8.5 and TigersFury:up() and Rake:remains() <= 9) or (TargetTimeToDie() >= Rake:remains() and Rake:remains() <= 3 and (Berserk:up() or Energy() > 70 or TigersFury:cooldown() + 0.8 >= Rake:remains())) then
        return Rake
    elseif OmenOfClarity:up() then
        return Shred
    elseif ComboPoints() >= 1 and SavageRoar:remains() <= 1 then
        return SavageRoar
	elseif T13SetPieces >= 4 and Stampede:up() and TigersFury:ready() then
		return Ravage
    elseif ComboPoints() == 5 and (TargetTimeToDie() <= 4 or (Rip:remains() >= 11 and SavageRoar:remains() >= 8)) then
        return FerociousBite
    elseif Stampede:up() and not OmenOfClarity:up() and TigersFury:up() then
        return Ravage
    elseif TigersFury:up() or Berserk:up() or (ComboPoints() < 5 and Rip:remains() <= 3) or (ComboPoints() == 0 and SavageRoar:remains() <= 2) or TigersFury:ready(3) or TargetTimeToDie() <= 8.5 or Energy() >= 78 then
        return Shred
    else
        return NoSpell
    end
end

local function CalculateBestChoiceAbilityBear()
	if Maul:ready() and Rage() >= 50 then
		abilityOffGCD = Maul
	elseif Enrage:ready() then
		abilityOffGCD = Enrage
	else
		abilityOffGCD = NoSpell
	end
    if FaerieFireFeral:ready() and FaerieFire:stack() < 3 and not (ExposeArmor:up() or SunderArmor:up()) then
        return FaerieFireFeral
	elseif not Claw.offgcd and Enrage:ready() and Rage() <= 80 then
		return Enrage
    elseif Claw.charge and FeralChargeBear:ready() and FeralChargeBear:inRange() then
        return FeralChargeBear
	elseif MangleBear:ready(0.4) then
		return MangleBear
	elseif Lacerate:remains() <= 2 then
		return Lacerate
	elseif Claw.roar and not (DemoralizingRoar:up() or DemoralizingShout:up() or Vindication:up() or ScarletFever:up()) then
		return DemoralizingRoar
	elseif Thrash:ready(0.4) then
		return Thrash
	elseif Lacerate:stack() < 3 then
		return Lacerate
	elseif Pulverize:known() and Lacerate:stack() == 3 and Pulverize:remains() <= 2 then
		return Pulverize
	else
        return Lacerate
    end
end

local function CalculateBestChoiceAbility()
    if Claw.motw and not StatBuffActive() then
        return MarkOfTheWild
	elseif CatForm:up() then
		return CalculateBestChoiceAbilityCat()
	elseif BearForm:up() then
		return CalculateBestChoiceAbilityBear()
	else
		return Pulverize:known() and BearForm or CatForm
	end
end

function events:PLAYER_EQUIPMENT_CHANGED()
	T13SetPieces = (GetItemInfo(GetInventoryItemID("player", 5) or 0) == "Deep Earth Raiment" and 1 or 0) +
		(GetItemInfo(GetInventoryItemID("player", 10) or 0) == "Deep Earth Grips" and 1 or 0) +
		(GetItemInfo(GetInventoryItemID("player", 1) or 0) == "Deep Earth Headpiece" and 1 or 0) +
		(GetItemInfo(GetInventoryItemID("player", 7) or 0) == "Deep Earth Legguards" and 1 or 0) +
		(GetItemInfo(GetInventoryItemID("player", 3) or 0) == "Deep Earth Spaulders" and 1 or 0)
end

function events:PLAYER_TARGET_CHANGED()
	hostileTargetted = not UnitCanAttack("player", "target")
end

function events:ACTIVE_TALENT_GROUP_CHANGED()
	playerHasBloodlettingGlyph = false
	for i = 1, GetNumGlyphSockets() do
		if select(4, GetGlyphSocketInfo(i)) == 54815 then
			playerHasBloodlettingGlyph = true
			break
		end
	end
end

function events:PLAYER_ENTERING_WORLD()
	events:ACTIVE_TALENT_GROUP_CHANGED()
	events:PLAYER_EQUIPMENT_CHANGED()
end

function events:ADDON_LOADED(name)
	if name == "Claw" then
		if not Claw.scale then
			print("It looks like this is your first time running Claw, why don't you take some time to familiarize yourself with the commands?")
			print("Type |cFFFFD000/claw|r for a list of commands.")
		end
		InitializeVariables()
		clawPanel:SetSize(64, 64)
		clawPanel:SetScale(Claw.scale)
		clawPanel:EnableMouse(not Claw.locked)
		clawPreviousPanel:SetSize(42, 42)
		clawPreviousPanel:SetScale(Claw.scale)
		clawPreviousPanel:EnableMouse(not Claw.locked)
		clawOffGCDPanel:SetSize(42, 42)
		clawOffGCDPanel:SetScale(Claw.scale)
		clawOffGCDPanel:EnableMouse(not Claw.locked)
        for i = 1, math.floor(3 / Claw.frequency) do targetHealthArray[i] = 0 end
		clawPanel:Show()
	end
end

function events:UNIT_SPELLCAST_SENT(srcName, spellName)
	if Claw.previous and srcName == "player" and spellName == abilityNow.name then
		abilityPrevious = abilityNow
	end
end

function events:COMBAT_LOG_EVENT_UNFILTERED(self, eventType, hideCaster, srcGUID, srcName, srcFlags, srcRaidFlags, dstGUID, dstName, dstFlags, dstRaidFlags, spellId, spellName, spellSchool, auraType)
	if Claw.previous then
		if eventType == "SPELL_CAST_SUCCESS" and srcGUID == UnitGUID("player") and spellId == abilityPrevious.spellId then
			clawPreviousPanel.icon:SetTexture(abilityPrevious.icon)
			clawPreviousPanel.border:SetTexture("Interface\\AddOns\\Claw\\border.blp")
			clawPreviousPanel:Show()
		elseif Claw.misseffect and eventType == "SPELL_MISSED" and srcGUID == UnitGUID("player") and spellId == abilityPrevious.spellId then
			clawPreviousPanel.border:SetTexture("Interface\\AddOns\\Claw\\misseffect.blp")
		end
	end
    if playerHasBloodlettingGlyph and srcGUID == UnitGUID("player") and dstGUID == UnitGUID("target") then
        if spellId == Rip.spellId and (eventType == "SPELL_AURA_APPLIED" or eventType == "SPELL_AURA_REFRESH") then
            ripExtendsLeft = 3
        elseif (spellId == Shred.spellId or spellId == MangleCat.spellId) and eventType == "SPELL_DAMAGE" then
            ripExtendsLeft = ripExtendsLeft - 1
        end
    end
end

clawPanel:SetScript("OnUpdate", function(self, elapsed)
    lastAbilityCalculation = lastAbilityCalculation + elapsed;
    if lastAbilityCalculation >= Claw.frequency then
		if UnitCanAttack("player", "target") and not UnitInVehicle("player") then
			if not hostileTargetted then
				clawPanel:EnableMouse(not Claw.locked)
				clawPreviousPanel:Hide()
				clawOffGCDPanel:Hide()
				for i = 1, #targetHealthArray do targetHealthArray[i] = UnitHealth("target") end
				hostileTargetted = true
			else
				targetHealthArray[#targetHealthArray + 1] = UnitHealth("target")
				table.remove(targetHealthArray, 1)
			end
			oldAbilityNow = abilityNow
			oldAbilityOffGCD = abilityOffGCD
			GCD = 1.5 - 1.5 * (GetCombatRatingBonus(20) * 0.01)
			currentTime = GetTime()
			abilityNow = CalculateBestChoiceAbility()
			if Claw.glow then
				abilityNow:glow(true)
				abilityOffGCD:glow(true)
			end
			if abilityNow ~= oldAbilityNow then
				clawPanel.icon:SetTexture(abilityNow.icon)
				if abilityNow.icon then
					clawPanel.icon:Show()
					clawPanel.border:Show()
				else
					clawPanel.icon:Hide()
					clawPanel.border:Hide()
				end
				if Claw.glow then
					oldAbilityNow:glow(false)
				end
			end
			if abilityOffGCD ~= oldAbilityOffGCD then
				clawOffGCDPanel.icon:SetTexture(abilityOffGCD.icon)
				if abilityOffGCD.icon then
					clawOffGCDPanel:Show()
				else
					clawOffGCDPanel:Hide()
				end
				if Claw.glow then
					oldAbilityOffGCD:glow(false)
				end
			end
		elseif hostileTargetted then
			if Claw.glow then
				abilityNow:glow(false)
				abilityOffGCD:glow(false)
			end
			abilityNow = NoSpell
			abilityOffGCD = NoSpell
			clawPanel.icon:Hide()
			clawPanel.border:Hide()
			clawPanel:EnableMouse(false)
			clawPreviousPanel:Hide()
			clawOffGCDPanel:Hide()
			for i = 1, #targetHealthArray do targetHealthArray[i] = 0 end
			hostileTargetted = false
		end
        lastAbilityCalculation = 0
    end
end)

clawPanel:SetScript("OnEvent", function(self, event, ...) events[event](self, ...) end)
for k in pairs(events) do
    clawPanel:RegisterEvent(k)
end

function SlashCmdList.Claw(msg, editbox)
    msg = { strsplit(" ", strlower(msg)) }
    if msg[1] == "locked" then
        if msg[2] then
			Claw.locked = msg[2] == "on" 
			clawPanel:EnableMouse(not Claw.locked and clawPanel.icon:IsVisible())
			clawPreviousPanel:EnableMouse(not Claw.locked)
			clawOffGCDPanel:EnableMouse(not Claw.locked)
		end
        print("Claw - Locked: " .. (Claw.locked and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "scale" then
        if msg[2] then
			Claw.scale = tonumber(msg[2]) or 1
			clawPanel:SetScale(Claw.scale)
			clawPreviousPanel:SetScale(Claw.scale)
			clawOffGCDPanel:SetScale(Claw.scale)
		end
		print("Claw - Scale: |cFFFFD000" .. Claw.scale .. "|r times")
	elseif msg[1] == "frequency" then
		Claw.frequency = tonumber(msg[2]) or 0.05
		print("Claw - Calculation frequency: Every |cFFFFD000" .. Claw.frequency .. "|r seconds")
	elseif msg[1] == "latency" then
		Claw.latency = tonumber(msg[2]) or 0.3
		print("Claw - Player reaction time: |cFFFFD000" .. Claw.latency .. "|r seconds")
    elseif msg[1] == "glow" then
        if msg[2] then
			Claw.glow = msg[2] == "on"
			if Claw.glow then
				abilityNow:glow(true)
				abilityOffGCD:glow(true)
			else
				abilityNow:glow(false)
				abilityOffGCD:glow(false)
			end
		end
        print("Claw - Glowing ability buttons: " .. (Claw.glow and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "previous" then
        if msg[2] then Claw.previous = msg[2] == "on" end
        print("Claw - Previous ability icon: " .. (Claw.previous and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "offgcd" then
        if msg[2] then Claw.offgcd = msg[2] == "on" end
        print("Claw - Off the global cooldown ability icon: " .. (Claw.offgcd and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "misseffect" then
        if msg[2] then Claw.misseffect = msg[2] == "on" end
        print("Claw - Red border around previous ability when it fails to hit: " .. (Claw.misseffect and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "charge" then
        if msg[2] then Claw.charge = msg[2] == "on" end
        print("Claw - Feral Charge (Bear and Cat): " .. (Claw.charge and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "berserk" then
        if msg[2] then Claw.berserk = msg[2] == "on" end
        print("Claw - Berserk on non-boss mobs (Cat only): " .. (Claw.berserk and "|cFF00C000On" or "|cFFC00000Off"))
	elseif msg[1] == "tfberserk" then
        if msg[2] then Claw.tfberserk = msg[2] == "on" end
        print("Claw - Tiger's Fury is macro'd to Berserk: " .. (Claw.tfberserk and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "roar" then
        if msg[2] then Claw.roar = msg[2] == "on" end
        print("Claw - Demoralizing Roar (Bear only): " .. (Claw.roar and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "motw" then
        if msg[2] then Claw.motw = msg[2] == "on" end
        print("Claw - Mark of the Wild: " .. (Claw.motw and "|cFF00C000On" or "|cFFC00000Off"))
    elseif msg[1] == "reset" then
		clawPanel:ClearAllPoints()
        clawPanel:SetPoint("CENTER", 0, -169)
		clawPreviousPanel:ClearAllPoints()
		clawPreviousPanel:SetPoint("BOTTOMRIGHT", clawPanel, "BOTTOMLEFT", -10, -5)
		clawOffGCDPanel:ClearAllPoints()
		clawOffGCDPanel:SetPoint("BOTTOMLEFT", clawPanel, "BOTTOMRIGHT", 10, -5)
		print("Claw - Position has been reset to default")
    else
        print("Claw (version: |cFFFFD000" .. GetAddOnMetadata("Claw", "Version") .. "|r) - Commands:")
        print("  /claw locked |cFF00C000on|r/|cFFC00000off|r - lock the Claw UI so that it can't be moved")
		print("  /claw scale |cFFFFD000[number]|r - set the scale of the Claw UI (default is 1 times)")
		print("  /claw frequency |cFFFFD000[number]|r - set the calculation frequency (default is every 0.05 seconds)")
		print("  /claw latency |cFFFFD000[number]|r - set the latency of the player's reaction time (default is 0.3 seconds)")
		print("  /claw glow |cFF00C000on|r/|cFFC00000off|r - enable glowing ability buttons on action bars")
		print("  /claw previous |cFF00C000on|r/|cFFC00000off|r - enable visibility of the previous ability icon")
		print("  /claw offgcd |cFF00C000on|r/|cFFC00000off|r - enable visibility of the off the global cooldown ability icon")
		print("  /claw misseffect |cFF00C000on|r/|cFFC00000off|r - enable red border around previous ability when it fails to hit")
		print("  /claw charge |cFF00C000on|r/|cFFC00000off|r - enable usage of Feral Charge (Bear and Cat)")
		print("  /claw berserk |cFF00C000on|r/|cFFC00000off|r - enable usage of Berserk on non-boss mobs (Cat only)")
		print("  /claw tfberserk |cFF00C000on|r/|cFFC00000off|r - Tiger's Fury is macro'd to Berserk")
		print("  /claw roar |cFF00C000on|r/|cFFC00000off|r - enable usage of Demoralizing Roar (Bear only)")
		print("  /claw motw |cFF00C000on|r/|cFFC00000off|r - enable usage of Mark of the Wild")
        print("  /claw reset - reset the location of the Claw UI to default")
		if Claw_Resources then
			print("For Claw Resources commands, please type |cFFFFD000/cres|r.")
		end
    end
end
